@@ -43,17 +43,18 @@ async def test_inject_100_simple_advertisements(benchmark: BenchmarkFixture) ->
     unsetup = scanner.async_setup()
     cancel = manager.async_register_scanner(scanner)
 
-    def run():
-        _address = switchbot_device.address
-        _rssi = switchbot_device_adv.rssi
-        _name = switchbot_device.name
-        _service_uuids = switchbot_device_adv.service_uuids
-        _service_data = switchbot_device_adv.service_data
-        _manufacturer_data = switchbot_device_adv.manufacturer_data
-        _tx_power = switchbot_device_adv.tx_power
-        _details = {"scanner_specific_data": "test"}
-        _now = monotonic_time_coarse()
+    _address = switchbot_device.address
+    _rssi = switchbot_device_adv.rssi
+    _name = switchbot_device.name
+    _service_uuids = switchbot_device_adv.service_uuids
+    _service_data = switchbot_device_adv.service_data
+    _manufacturer_data = switchbot_device_adv.manufacturer_data
+    _tx_power = switchbot_device_adv.tx_power
+    _details = {"scanner_specific_data": "test"}
+    _now = monotonic_time_coarse()
 
+    @benchmark
+    def run():
         for _ in range(100):
             scanner._async_on_advertisement(
                 _address,
@@ -67,8 +68,6 @@ async def test_inject_100_simple_advertisements(benchmark: BenchmarkFixture) ->
                 _now,
             )
 
-    benchmark(run)
-
     cancel()
     unsetup()
 
@@ -89,7 +88,7 @@ async def test_inject_100_complex_advertisements(benchmark: BenchmarkFixture) ->
         local_name="wohand",
         service_uuids=["050a021a-0000-1000-8000-00805f9b34fb"],
         service_data={"050a021a-0000-1000-8000-00805f9b34fb": b"\n\xff"},
-        manufacturer_data={i: b"\x01" for i in range(100)},
+        manufacturer_data=dict.fromkeys(range(100), b"\x01"),
         rssi=-100,
     )
 
@@ -100,17 +99,18 @@ async def test_inject_100_complex_advertisements(benchmark: BenchmarkFixture) ->
     unsetup = scanner.async_setup()
     cancel = manager.async_register_scanner(scanner)
 
-    def run():
-        _address = switchbot_device.address
-        _rssi = switchbot_device_adv.rssi
-        _name = switchbot_device.name
-        _service_uuids = switchbot_device_adv.service_uuids
-        _service_data = switchbot_device_adv.service_data
-        _manufacturer_data = switchbot_device_adv.manufacturer_data
-        _tx_power = switchbot_device_adv.tx_power
-        _details = {"scanner_specific_data": "test"}
-        _now = monotonic_time_coarse()
+    _address = switchbot_device.address
+    _rssi = switchbot_device_adv.rssi
+    _name = switchbot_device.name
+    _service_uuids = switchbot_device_adv.service_uuids
+    _service_data = switchbot_device_adv.service_data
+    _manufacturer_data = switchbot_device_adv.manufacturer_data
+    _tx_power = switchbot_device_adv.tx_power
+    _details = {"scanner_specific_data": "test"}
+    _now = monotonic_time_coarse()
 
+    @benchmark
+    def run():
         for _ in range(100):
             scanner._async_on_advertisement(
                 _address,
@@ -124,8 +124,6 @@ async def test_inject_100_complex_advertisements(benchmark: BenchmarkFixture) ->
                 _now,
             )
 
-    benchmark(run)
-
     cancel()
     unsetup()
 
@@ -161,16 +159,17 @@ async def test_inject_100_different_advertisements(benchmark: BenchmarkFixture)
     unsetup = scanner.async_setup()
     cancel = manager.async_register_scanner(scanner)
 
-    def run():
-        _address = switchbot_device.address
-        _rssi = switchbot_device_adv.rssi
-        _name = switchbot_device.name
-        _service_uuids = switchbot_device_adv.service_uuids
-        _service_data = switchbot_device_adv.service_data
-        _tx_power = switchbot_device_adv.tx_power
-        _details = {"scanner_specific_data": "test"}
-        _now = monotonic_time_coarse()
+    _address = switchbot_device.address
+    _rssi = switchbot_device_adv.rssi
+    _name = switchbot_device.name
+    _service_uuids = switchbot_device_adv.service_uuids
+    _service_data = switchbot_device_adv.service_data
+    _tx_power = switchbot_device_adv.tx_power
+    _details = {"scanner_specific_data": "test"}
+    _now = monotonic_time_coarse()
 
+    @benchmark
+    def run():
         for adv in advs:
             scanner._async_on_advertisement(
                 _address,
@@ -184,8 +183,6 @@ async def test_inject_100_different_advertisements(benchmark: BenchmarkFixture)
                 _now,
             )
 
-    benchmark(run)
-
     cancel()
     unsetup()
 
@@ -223,16 +220,17 @@ async def test_inject_100_different_manufacturer_data(
     unsetup = scanner.async_setup()
     cancel = manager.async_register_scanner(scanner)
 
-    def run():
-        _address = switchbot_device.address
-        _rssi = switchbot_device_adv.rssi
-        _name = switchbot_device.name
-        _service_uuids = switchbot_device_adv.service_uuids
-        _service_data = switchbot_device_adv.service_data
-        _tx_power = switchbot_device_adv.tx_power
-        _details = {"scanner_specific_data": "test"}
-        _now = monotonic_time_coarse()
+    _address = switchbot_device.address
+    _rssi = switchbot_device_adv.rssi
+    _name = switchbot_device.name
+    _service_uuids = switchbot_device_adv.service_uuids
+    _service_data = switchbot_device_adv.service_data
+    _tx_power = switchbot_device_adv.tx_power
+    _details = {"scanner_specific_data": "test"}
+    _now = monotonic_time_coarse()
 
+    @benchmark
+    def run():
         for adv in advs:
             scanner._async_on_advertisement(
                 _address,
@@ -246,8 +244,6 @@ async def test_inject_100_different_manufacturer_data(
                 _now,
             )
 
-    benchmark(run)
-
     cancel()
     unsetup()
 
@@ -285,16 +281,17 @@ async def test_inject_100_different_service_data(
     unsetup = scanner.async_setup()
     cancel = manager.async_register_scanner(scanner)
 
-    def run():
-        _address = switchbot_device.address
-        _rssi = switchbot_device_adv.rssi
-        _name = switchbot_device.name
-        _service_uuids = switchbot_device_adv.service_uuids
-        _service_data = switchbot_device_adv.service_data
-        _tx_power = switchbot_device_adv.tx_power
-        _details = {"scanner_specific_data": "test"}
-        _now = monotonic_time_coarse()
+    _address = switchbot_device.address
+    _rssi = switchbot_device_adv.rssi
+    _name = switchbot_device.name
+    _service_uuids = switchbot_device_adv.service_uuids
+    _service_data = switchbot_device_adv.service_data
+    _tx_power = switchbot_device_adv.tx_power
+    _details = {"scanner_specific_data": "test"}
+    _now = monotonic_time_coarse()
 
+    @benchmark
+    def run():
         for adv in advs:
             scanner._async_on_advertisement(
                 _address,
@@ -308,8 +305,6 @@ async def test_inject_100_different_service_data(
                 _now,
             )
 
-    benchmark(run)
-
     cancel()
     unsetup()
 
@@ -624,16 +619,17 @@ async def test_inject_100_rotating_manufacturer_data(
         )
         advs.append(sensorpush_device_adv)
 
-    def run():
-        _address = sensor_push_device.address
-        _rssi = sensorpush_device_adv.rssi
-        _name = sensor_push_device.name
-        _service_uuids = sensorpush_device_adv.service_uuids
-        _service_data = sensorpush_device_adv.service_data
-        _tx_power = sensorpush_device_adv.tx_power
-        _details = {"scanner_specific_data": "test"}
-        _now = monotonic_time_coarse()
+    _address = sensor_push_device.address
+    _rssi = sensorpush_device_adv.rssi
+    _name = sensor_push_device.name
+    _service_uuids = sensorpush_device_adv.service_uuids
+    _service_data = sensorpush_device_adv.service_data
+    _tx_power = sensorpush_device_adv.tx_power
+    _details = {"scanner_specific_data": "test"}
+    _now = monotonic_time_coarse()
 
+    @benchmark
+    def run():
         for adv in advs:
             scanner._async_on_advertisement(
                 _address,
@@ -647,7 +643,116 @@ async def test_inject_100_rotating_manufacturer_data(
                 _now,
             )
 
-    benchmark(run)
+    cancel()
+    unsetup()
+
+
+@pytest.mark.usefixtures("enable_bluetooth")
+@pytest.mark.asyncio
+async def test_filter_unwanted_apple_advs(benchmark: BenchmarkFixture) -> None:
+    """Test filtering unwanted apple data."""
+    manager = get_manager()
+
+    device = generate_ble_device(
+        "44:44:33:11:23:45",
+        "beacon",
+        {},
+        rssi=-100,
+    )
+    device_adv = generate_advertisement_data(
+        local_name="beacon",
+        service_uuids=[],
+        service_data={},
+        manufacturer_data={76: b"\xff"},
+        rssi=-100,
+    )
+
+    connector = HaBluetoothConnector(
+        MockBleakClient, "mock_bleak_client", lambda: False
+    )
+    scanner = BaseHaRemoteScanner("esp32", "esp32", connector, True)
+    unsetup = scanner.async_setup()
+    cancel = manager.async_register_scanner(scanner)
+    _address = device.address
+    _rssi = device.rssi
+    _name = device.name
+    _service_uuids = device_adv.service_uuids
+    _service_data = device_adv.service_data
+    _manufacturer_data = device_adv.manufacturer_data
+    _tx_power = device_adv.tx_power
+    _details = {"scanner_specific_data": "test"}
+    _now = monotonic_time_coarse()
+
+    @benchmark
+    def run():
+        for _ in range(100):
+            scanner._async_on_advertisement(
+                _address,
+                _rssi,
+                _name,
+                _service_uuids,
+                _service_data,
+                _manufacturer_data,
+                _tx_power,
+                _details,
+                _now,
+            )
+
+    cancel()
+    unsetup()
+
+
+@pytest.mark.usefixtures("enable_bluetooth")
+@pytest.mark.asyncio
+async def test_filter_wanted_apple_advs(benchmark: BenchmarkFixture) -> None:
+    """Test filtering wanted apple data."""
+    manager = get_manager()
+
+    device = generate_ble_device(
+        "44:44:33:11:23:45",
+        "beacon",
+        {},
+        rssi=-100,
+    )
+    device_adv = generate_advertisement_data(
+        local_name="beacon",
+        service_uuids=[],
+        service_data={},
+        manufacturer_data={76: b"\x02"},
+        rssi=-100,
+    )
+
+    connector = HaBluetoothConnector(
+        MockBleakClient, "mock_bleak_client", lambda: False
+    )
+    scanner = BaseHaRemoteScanner("esp32", "esp32", connector, True)
+    unsetup = scanner.async_setup()
+    cancel = manager.async_register_scanner(scanner)
+
+    _address = device.address
+    _rssi = device.rssi
+    _name = device.name
+    _service_uuids = device_adv.service_uuids
+    _service_data = device_adv.service_data
+    _manufacturer_data = device_adv.manufacturer_data
+    _tx_power = device_adv.tx_power
+    _details = {"scanner_specific_data": "test"}
+    _now = monotonic_time_coarse()
+
+    @benchmark
+    def run():
+        for _ in range(100):
+            scanner._async_on_advertisement(
+                _address,
+                _rssi,
+                _name,
+                _service_uuids,
+                _service_data,
+                _manufacturer_data,
+                _tx_power,
+                _details,
+                _now,
+            )
 
     cancel()
     unsetup()
